@@ -48,6 +48,35 @@ type MemoryMetrics struct {
 	Total, Used, Available, SwapTotal, SwapUsed uint64
 }
 
+type EventThrottler struct {
+	timer       *time.Timer
+	gracePeriod time.Duration
+
+	C chan struct{}
+}
+
+func NewEventThrottler(gracePeriod time.Duration) *EventThrottler {
+	return &EventThrottler{
+		timer:       nil,
+		gracePeriod: gracePeriod,
+		C:           make(chan struct{}, 1),
+	}
+}
+
+func (e *EventThrottler) Notify() {
+	if e.timer != nil {
+		return
+	}
+
+	e.timer = time.AfterFunc(e.gracePeriod, func() {
+		e.timer = nil
+		select {
+		case e.C <- struct{}{}:
+		default:
+		}
+	})
+}
+
 var (
 	cpu1Gauge, cpu2Gauge, gpuGauge, aneGauge        *w.Gauge
 	TotalPowerChart                                 *w.BarChart
@@ -61,6 +90,18 @@ var (
 	updateInterval                                  = 1000
 )
 
+var (
+	dataRegex   = regexp.MustCompile(`(?m)^\s*(\S.*?)\s+(\d+)\s+(\d+\.\d+)\s+\d+\.\d+\s+`)
+	outRegex    = regexp.MustCompile(`out:\s*([\d.]+)\s*packets/s,\s*([\d.]+)\s*bytes/s`)
+	inRegex     = regexp.MustCompile(`in:\s*([\d.]+)\s*packets/s,\s*([\d.]+)\s*bytes/s`)
+	readRegex   = regexp.MustCompile(`read:\s*([\d.]+)\s*ops/s\s*([\d.]+)\s*KBytes/s`)
+	writeRegex  = regexp.MustCompile(`write:\s*([\d.]+)\s*ops/s\s*([\d.]+)\s*KBytes/s`)
+	residencyRe = regexp.MustCompile(`(\w+-Cluster)\s+HW active residency:\s+(\d+\.\d+)%`)
+	frequencyRe = regexp.MustCompile(`(\w+-Cluster)\s+HW active frequency:\s+(\d+)\s+MHz`)
+	re          = regexp.MustCompile(`GPU\s*(HW)?\s*active\s*(residency|frequency):\s+(\d+\.\d+)%?`)
+	freqRe      = regexp.MustCompile(`(\d+)\s*MHz:\s*(\d+)%`)
+)
+
 func setupUI() {
 	appleSiliconModel := getSOCInfo()
 	modelText = w.NewParagraph()
@@ -161,7 +202,6 @@ func setupGrid() {
 
 func switchGridLayout() {
 	if currentGridLayout == "default" {
-		ui.Clear()
 		newGrid := ui.NewGrid()
 		newGrid.Set(
 			ui.NewRow(1.0/2, // This row now takes half the height of the grid
@@ -184,9 +224,7 @@ func switchGridLayout() {
 		newGrid.SetRect(0, 0, termWidth, termHeight)
 		grid = newGrid
 		currentGridLayout = "alternative"
-		ui.Render(grid)
 	} else {
-		ui.Clear()
 		newGrid := ui.NewGrid()
 		newGrid.Set(
 			ui.NewRow(1.0/2,
@@ -207,7 +245,6 @@ func switchGridLayout() {
 		newGrid.SetRect(0, 0, termWidth, termHeight)
 		grid = newGrid
 		currentGridLayout = "default"
-		ui.Render(grid)
 	}
 }
 
@@ -222,7 +259,7 @@ func main() {
 		err                   error
 		setColor, setInterval bool
 	)
-	version := "v0.1.7"
+	version := "v0.1.8"
 	for i := 1; i < len(os.Args); i++ {
 		switch os.Args[i] {
 		case "--help", "-h":
@@ -340,21 +377,24 @@ func main() {
 	appleSiliconModel := getSOCInfo()
 	go collectMetrics(done, cpuMetricsChan, gpuMetricsChan, netdiskMetricsChan, processMetricsChan, appleSiliconModel["name"].(string))
 	lastUpdateTime = time.Now()
+	needRender := NewEventThrottler(time.Duration(updateInterval/2) * time.Millisecond)
 	go func() {
 		for {
 			select {
 			case cpuMetrics := <-cpuMetricsChan:
 				updateCPUUI(cpuMetrics)
 				updateTotalPowerChart(cpuMetrics.PackageW)
-				ui.Render(grid)
+				needRender.Notify()
 			case gpuMetrics := <-gpuMetricsChan:
 				updateGPUUI(gpuMetrics)
-				ui.Render(grid)
+				needRender.Notify()
 			case netdiskMetrics := <-netdiskMetricsChan:
 				updateNetDiskUI(netdiskMetrics)
-				ui.Render(grid)
+				needRender.Notify()
 			case processMetrics := <-processMetricsChan:
 				updateProcessUI(processMetrics)
+				needRender.Notify()
+			case <-needRender.C:
 				ui.Render(grid)
 			case <-quit:
 				close(done)
@@ -401,10 +441,10 @@ func main() {
 }
 
 func setupLogfile() (*os.File, error) {
-	if err := os.MkdirAll("logs", 0755); err != nil {
+	if err := os.MkdirAll("/var/log", 0755); err != nil {
 		return nil, fmt.Errorf("failed to make the log directory: %v", err)
 	}
-	logfile, err := os.OpenFile("logs/mactop.log", os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0660)
+	logfile, err := os.OpenFile("/var/log/mactop.log", os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0660)
 	if err != nil {
 		return nil, fmt.Errorf("failed to open log file: %v", err)
 	}
@@ -477,7 +517,6 @@ func updateTotalPowerChart(newPowerValue float64) {
 		}
 		powerValues = nil
 		lastUpdateTime = currentTime
-		ui.Render(TotalPowerChart)
 	}
 }
 
@@ -495,8 +534,6 @@ func updateCPUUI(cpuMetrics CPUMetrics) {
 	memoryMetrics := getMemoryMetrics()
 	memoryGauge.Title = fmt.Sprintf("Memory Usage: %.2f GB / %.2f GB (Swap: %.2f/%.2f GB)", float64(memoryMetrics.Used)/1024/1024/1024, float64(memoryMetrics.Total)/1024/1024/1024, float64(memoryMetrics.SwapUsed)/1024/1024/1024, float64(memoryMetrics.SwapTotal)/1024/1024/1024)
 	memoryGauge.Percent = int((float64(memoryMetrics.Used) / float64(memoryMetrics.Total)) * 100)
-	ui.Render(grid)
-	ui.Render(cpu1Gauge, cpu2Gauge, gpuGauge, aneGauge, memoryGauge, modelText, PowerChart)
 }
 
 func updateGPUUI(gpuMetrics GPUMetrics) {
@@ -520,12 +557,10 @@ func updateProcessUI(processMetrics []ProcessMetrics) {
 	for _, pm := range processMetrics {
 		ProcessInfo.Text += fmt.Sprintf("%d - %s: %.2f ms/s\n", pm.ID, pm.Name, pm.CPUUsage)
 	}
-	ui.Render(ProcessInfo)
 }
 
 func parseProcessMetrics(powermetricsOutput string, processMetrics []ProcessMetrics) []ProcessMetrics {
 	lines := strings.Split(powermetricsOutput, "\n")
-	dataRegex := regexp.MustCompile(`(?m)^\s*(\S.*?)\s+(\d+)\s+(\d+\.\d+)\s+\d+\.\d+\s+`)
 	seen := make(map[int]bool) // Map to track seen process IDs
 	for _, line := range lines {
 		matches := dataRegex.FindStringSubmatch(line)
@@ -554,8 +589,7 @@ func parseProcessMetrics(powermetricsOutput string, processMetrics []ProcessMetr
 }
 
 func parseActivityMetrics(powermetricsOutput string, netdiskMetrics NetDiskMetrics) NetDiskMetrics {
-	outRegex := regexp.MustCompile(`out:\s*([\d.]+)\s*packets/s,\s*([\d.]+)\s*bytes/s`)
-	inRegex := regexp.MustCompile(`in:\s*([\d.]+)\s*packets/s,\s*([\d.]+)\s*bytes/s`)
+
 	outMatches := outRegex.FindStringSubmatch(powermetricsOutput)
 	inMatches := inRegex.FindStringSubmatch(powermetricsOutput)
 	if len(outMatches) == 3 {
@@ -566,8 +600,7 @@ func parseActivityMetrics(powermetricsOutput string, netdiskMetrics NetDiskMetri
 		netdiskMetrics.InPacketsPerSec, _ = strconv.ParseFloat(inMatches[1], 64)
 		netdiskMetrics.InBytesPerSec, _ = strconv.ParseFloat(inMatches[2], 64)
 	}
-	readRegex := regexp.MustCompile(`read:\s*([\d.]+)\s*ops/s\s*([\d.]+)\s*KBytes/s`)
-	writeRegex := regexp.MustCompile(`write:\s*([\d.]+)\s*ops/s\s*([\d.]+)\s*KBytes/s`)
+
 	readMatches := readRegex.FindStringSubmatch(powermetricsOutput)
 	writeMatches := writeRegex.FindStringSubmatch(powermetricsOutput)
 	if len(readMatches) == 3 {
@@ -587,8 +620,7 @@ func parseCPUMetrics(powermetricsOutput string, cpuMetrics CPUMetrics, modelName
 	pCores := []int{}
 	var eClusterActiveSum, pClusterActiveSum, eClusterFreqSum, pClusterFreqSum float64
 	var eClusterCount, pClusterCount, eClusterActiveTotal, pClusterActiveTotal, eClusterFreqTotal, pClusterFreqTotal int
-	residencyRe := regexp.MustCompile(`(\w+-Cluster)\s+HW active residency:\s+(\d+\.\d+)%`)
-	frequencyRe := regexp.MustCompile(`(\w+-Cluster)\s+HW active frequency:\s+(\d+)\s+MHz`)
+
 	if modelName == "Apple M3 Max" || modelName == "Apple M2 Max" { // For the M3/M2 Max, we need to manually parse the CPU Usage from the powermetrics output (as current bug in Apple's powermetrics)
 		for _, line := range lines {
 
@@ -808,8 +840,7 @@ func max(nums ...int) int {
 }
 
 func parseGPUMetrics(powermetricsOutput string, gpuMetrics GPUMetrics) GPUMetrics {
-	re := regexp.MustCompile(`GPU\s*(HW)?\s*active\s*(residency|frequency):\s+(\d+\.\d+)%?`)
-	freqRe := regexp.MustCompile(`(\d+)\s*MHz:\s*(\d+)%`)
+
 	lines := strings.Split(powermetricsOutput, "\n")
 
 	for _, line := range lines {
@@ -863,48 +894,6 @@ func getSOCInfo() map[string]interface{} {
 		"gpu_core_count": getGPUCores(),
 	}
 
-	switch socInfo["name"] {
-	case "Apple M1 Max":
-		socInfo["cpu_max_power"] = 30
-		socInfo["gpu_max_power"] = 60
-	case "Apple M1 Pro":
-		socInfo["cpu_max_power"] = 30
-		socInfo["gpu_max_power"] = 30
-	case "Apple M1":
-		socInfo["cpu_max_power"] = 20
-		socInfo["gpu_max_power"] = 20
-	case "Apple M1 Ultra":
-		socInfo["cpu_max_power"] = 60
-		socInfo["gpu_max_power"] = 120
-	case "Apple M2":
-		socInfo["cpu_max_power"] = 25
-		socInfo["gpu_max_power"] = 15
-	default:
-		socInfo["cpu_max_power"] = 20
-		socInfo["gpu_max_power"] = 20
-	}
-
-	switch socInfo["name"] {
-	case "Apple M1 Max":
-		socInfo["cpu_max_bw"] = 250
-		socInfo["gpu_max_bw"] = 400
-	case "Apple M1 Pro":
-		socInfo["cpu_max_bw"] = 200
-		socInfo["gpu_max_bw"] = 200
-	case "Apple M1":
-		socInfo["cpu_max_bw"] = 70
-		socInfo["gpu_max_bw"] = 70
-	case "Apple M1 Ultra":
-		socInfo["cpu_max_bw"] = 500
-		socInfo["gpu_max_bw"] = 800
-	case "Apple M2":
-		socInfo["cpu_max_bw"] = 100
-		socInfo["gpu_max_bw"] = 100
-	default:
-		socInfo["cpu_max_bw"] = 70
-		socInfo["gpu_max_bw"] = 70
-	}
-
 	return socInfo
 }
 
